@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.17 2001/11/27 07:27:32 bagder Exp $
+ * $Id: setup.h,v 1.21 2002/02/07 14:47:41 bagder Exp $
  *****************************************************************************/
 
 
@@ -42,7 +42,7 @@
 #else
 #ifdef WIN32
 /* hand-modified win32 config.h! */
-#include "../config-win32.h"
+#include "config-win32.h"
 #endif
 #ifdef macintosh
 /* hand-modified MacOS config.h! */
@@ -52,7 +52,8 @@
 #endif
 
 #ifndef __cplusplus        /* (rabe) */
-typedef char bool;
+typedef unsigned char bool;
+#define typedef_bool
 #endif                     /* (rabe) */
 
 #ifdef NEED_REENTRANT
@@ -93,11 +94,12 @@ defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
 #ifdef	VMS
 #include "../include/curl/stdcheaders.h"
 #else
-#include "curl/stdcheaders.h"
+#include <curl/stdcheaders.h>
 #endif
+
 #else
 #ifdef _AIX
-#include "curl/stdcheaders.h"
+#include <curl/stdcheaders.h>
 #endif
 #endif
 
